@@ -1,4 +1,3 @@
-from datetime import datetime
 from os import get_terminal_size
 from os.path import isdir
 from shutil import move
@@ -8,14 +7,13 @@ from typing import Tuple
 from typing import Union
 
 from faapi import FAAPI
+from faapi import Journal
 from faapi import Sub
 
 from .database import Connection
+from .database import keys_journals
 from .database import keys_submissions
-from .database import select_all
 from .download import submission_download
-from .download import user_clean_name
-from .download import user_download
 from .settings import setting_write
 
 
@@ -27,12 +25,25 @@ def files_folder_move(db: Connection, folder_old: str, folder_new: str):
         print("Done")
 
 
-def users_download(api: FAAPI, db: Connection, users: List[str], folders: List[str]):
-    for user, folder in ((u, f) for u in users for f in folders):
-        print(f"Downloading: {user}/{folder}")
-        tot, fail = user_download(api, db, user, folder)
-        print("Submissions downloaded:", tot)
-        print("Submissions failed:", fail)
+def journal_make(id_: Union[int, str], author: str,
+                 title: str, date: str, content: str = ""
+                 ) -> Journal:
+    assert isinstance(id_, int) or (isinstance(id_, str) and id_.isdigit())
+    assert int(id_) > 0
+    assert isinstance(author, str) and author
+    assert isinstance(title, str) and title
+    assert isinstance(date, str) and date
+    assert isinstance(content, str)
+
+    journal = Journal()
+
+    journal.id = int(id_)
+    journal.author = author
+    journal.title = title
+    journal.date = date
+    journal.content = content
+
+    return journal
 
 
 def submissions_download(api: FAAPI, db: Connection, sub_ids: List[str]):
@@ -43,27 +54,6 @@ def submissions_download(api: FAAPI, db: Connection, sub_ids: List[str]):
         submission_download(api, db, sub_id)
 
 
-def users_update(api: FAAPI, db: Connection, users: List[str] = None, folders: List[str] = None):
-    tot: int = 0
-    fail: int = 0
-    for user, user_folders in select_all(db, "USERS", ["USERNAME", "FOLDERS"]):
-        if users and user not in users:
-            continue
-        for folder in user_folders.split(","):
-            if folders and folder not in folders:
-                continue
-            if folder.lower().startswith("mentions"):
-                print(f"Unsupported: {user}/{folder}")
-                continue
-            print(f"Downloading: {user}/{folder}")
-            tot_tmp, fail_tmp = user_download(api, db, user, folder, 1)
-            tot += tot_tmp
-            fail += fail_tmp
-    print("Submissions downloaded:", tot)
-    print("Submissions failed:", fail)
-    setting_write(db, "LASTUPDATE", str(datetime.now().timestamp()))
-
-
 def submission_make(id_: Union[int, str], author: str, title: str,
                     date: str, category: str, species: str,
                     gender: str, rating: str, tags: str = "",
@@ -106,20 +96,79 @@ def submission_make(id_: Union[int, str], author: str, title: str,
     return sub, sub_file
 
 
+def journals_search(db: Connection,
+                    limit: List[Union[str, int]] = None, offset: List[Union[str, int]] = None,
+                    order: List[str] = None, author: List[str] = None, title: List[str] = None,
+                    date: List[str] = None, content: List[str] = None
+                    ) -> List[tuple]:
+    order = [] if order is None else order
+    author = [] if author is None else list(map(str.lower, author))
+    title = [] if title is None else list(map(str.lower, title))
+    date = [] if date is None else list(map(str.lower, date))
+    content = [] if content is None else list(map(str.lower, content))
+
+    assert any((author, title, date, content))
+
+    wheres: List[str] = [
+        " OR ".join(["UDATE like ?"] * len(date)),
+        " OR ".join(['replace(lower(AUTHOR), "_", "") like ?'] * len(author)),
+        " OR ".join(["lower(TITLE) like ?"] * len(title)),
+        " OR ".join(["lower(content) like ?"] * len(content))
+    ]
+
+    wheres_str = " AND ".join(map(lambda p: "(" + p + ")", filter(len, wheres)))
+    order_str: str = f"ORDER BY {','.join(order)}" if order else ""
+    limit_str: str = f"LIMIT {int(limit[0])}" if limit is not None else ""
+    offset_str: str = f"OFFSET {int(offset[0])}" if offset is not None else ""
+
+    return db.execute(
+        f"""SELECT * FROM JOURNALS WHERE {wheres_str} {order_str} {limit_str} {offset_str}""",
+        date + author + title + content
+    ).fetchall()
+
+
+def journals_print(journals: List[tuple]):
+    space_id: int = 10
+    space_user: int = 10
+    space_date: int = 10
+    space_term: int = 10000
+    try:
+        space_term = get_terminal_size()[0]
+    except IOError:
+        pass
+
+    index_id: int = keys_journals.index("ID")
+    index_user: int = keys_journals.index("AUTHOR")
+    index_date: int = keys_journals.index("UDATE")
+    index_title: int = keys_journals.index("TITLE")
+
+    print(f"{'ID':^{space_id}} | {'User':^{space_user}} | {'Date':^{space_date}} | Title")
+    for journal in journals:
+        print(
+            f"{str(journal[index_id])[:space_id].zfill(space_id)} | " +
+            f"{journal[index_user][:space_user]:<{space_user}} | " +
+            f"{journal[index_date][:space_date]:<{space_date}} | " +
+            journal[index_title][:(space_term - space_id - space_user - space_date - 10)]
+        )
+
+
 def submissions_search(db: Connection,
-                       author: List[str] = None, title: List[str] = None, date: List[str] = None,
-                       description: List[str] = None, tags: List[str] = None, category: List[str] = None,
-                       species: List[str] = None, gender: List[str] = None, rating: List[str] = None
+                       limit: List[Union[str, int]] = None, offset: List[Union[str, int]] = None,
+                       order: List[str] = None, author: List[str] = None, title: List[str] = None,
+                       date: List[str] = None, description: List[str] = None, tags: List[str] = None,
+                       category: List[str] = None, species: List[str] = None, gender: List[str] = None,
+                       rating: List[str] = None
                        ) -> List[tuple]:
-    author = [] if author is None else list(map(user_clean_name, author))
-    title = [] if title is None else title
-    date = [] if date is None else date
-    description = [] if description is None else description
-    tags = [] if tags is None else tags
-    category = [] if category is None else category
-    species = [] if species is None else species
-    gender = [] if gender is None else gender
-    rating = [] if rating is None else rating
+    order = [] if order is None else order
+    author = [] if author is None else list(map(str.lower, author))
+    title = [] if title is None else list(map(str.lower, title))
+    date = [] if date is None else list(map(str.lower, date))
+    description = [] if description is None else list(map(str.lower, description))
+    tags = [] if tags is None else list(map(str.lower, tags))
+    category = [] if category is None else list(map(str.lower, category))
+    species = [] if species is None else list(map(str.lower, species))
+    gender = [] if gender is None else list(map(str.lower, gender))
+    rating = [] if rating is None else list(map(str.lower, rating))
 
     assert any((author, title, date, description, tags, category, species, gender, rating))
 
@@ -135,15 +184,18 @@ def submissions_search(db: Connection,
         " OR ".join(["lower(DESCRIPTION) like ?"] * len(description))
     ]
 
-    wheres_str = " AND ".join(map(lambda p: "(" + p + ")", filter(len, wheres)))
+    wheres_str: str = " AND ".join(map(lambda p: "(" + p + ")", filter(len, wheres)))
+    order_str: str = f"ORDER BY {','.join(order)}" if order else ""
+    limit_str: str = f"LIMIT {int(limit[0])}" if limit is not None else ""
+    offset_str: str = f"OFFSET {int(offset[0])}" if offset is not None else ""
 
     return db.execute(
-        f"""SELECT * FROM SUBMISSIONS WHERE {wheres_str}""",
+        f"""SELECT * FROM SUBMISSIONS WHERE {wheres_str} {order_str} {limit_str} {offset_str}""",
         date + rating + gender + species + category + author + title + tags + description
     ).fetchall()
 
 
-def submissions_print(subs: List[tuple], sort: bool = True):
+def submissions_print(subs: List[tuple]):
     space_id: int = 10
     space_user: int = 10
     space_date: int = 10
@@ -158,9 +210,6 @@ def submissions_print(subs: List[tuple], sort: bool = True):
     index_date: int = keys_submissions.index("UDATE")
     index_title: int = keys_submissions.index("TITLE")
 
-    if sort:
-        subs.sort(key=lambda s: (s[index_user], s[index_date]))
-
     print(f"{'ID':^{space_id}} | {'User':^{space_user}} | {'Date':^{space_date}} | Title")
     for sub in subs:
         print(
