@@ -1,23 +1,24 @@
 from os.path import join as path_join
 from sqlite3 import Connection
+from sqlite3 import Cursor
 from sqlite3 import connect as sqlite3_connect
-from typing import Iterator
 from typing import List
 from typing import Union
 
 from .__version__ import __database_version__
 
 # Entries guide - USERS
-# v3.0        v3.1
-# 0 USERNAME  USERNAME
-# 1 FOLDERS   FOLDERS
-# 2 GALLERY   GALLERY
-# 3 SCRAPS    SCRAPS
-# 4 FAVORITES FAVORITES
-# 5 EXTRAS    MENTIONS
+# v3.0        v3.1      v3.2
+# 0 USERNAME  USERNAME  USERNAME
+# 1 FOLDERS   FOLDERS   FOLDERS
+# 2 GALLERY   GALLERY   GALLERY
+# 3 SCRAPS    SCRAPS    SCRAPS
+# 4 FAVORITES FAVORITES FAVORITES
+# 5 EXTRAS    MENTIONS  MENTIONS
+# 6                     JOURNALS
 
 # Entries guide - SUBMISSIONS
-# v3.1
+# v3.2
 # 0  ID
 # 1  AUTHOR
 # 2  TITLE
@@ -34,6 +35,14 @@ from .__version__ import __database_version__
 # 13 LOCATION
 # 14 SERVER
 
+# Entries guide - JOURNALS
+# v3.2
+# 0 ID
+# 1 AUTHOR
+# 2 TITLE
+# 3 UDATE
+# 4 CONTENT
+
 
 keys_submissions: List[str] = [
     "ID", "AUTHOR", "TITLE",
@@ -43,24 +52,26 @@ keys_submissions: List[str] = [
     "FILESAVED"
 ]
 
+keys_journals: List[str] = [
+    "ID", "AUTHOR", "TITLE",
+    "UDATE", "CONTENT"
+]
+
 keys_users: List[str] = [
     "USERNAME", "FOLDERS",
     "GALLERY", "SCRAPS",
-    "FAVORITES", "MENTIONS"
+    "FAVORITES", "MENTIONS",
+    "JOURNALS"
 ]
 
 
 def tiered_path(id_: Union[int, str], tiers: int = 5, depth: int = 2) -> str:
     assert isinstance(id_, int) or (isinstance(id_, str) and id_.isdigit())
-    assert isinstance(tiers, int)
-    assert isinstance(depth, int)
+    assert isinstance(tiers, int) and tiers > 0
+    assert isinstance(depth, int) and depth > 0
 
-    id_str: str = str(id_).lstrip("0").zfill(tiers * depth)
-    id_tiered: List[str] = []
-    for n in range(0, tiers * depth, depth):
-        id_tiered.append(id_str[n:n + depth])
-
-    return path_join(*id_tiered)
+    id_str: str = str(int(id_)).zfill(tiers * depth)
+    return path_join(*[id_str[n:n + depth] for n in range(0, tiers * depth, depth)])
 
 
 def connect_database(db_name: str) -> Connection:
@@ -77,14 +88,14 @@ def insert(db: Connection, table: str, keys: List[str], values: List[Union[int,
 
 
 def select(db: Connection, table: str, select_fields: List[str], key: str, key_value: Union[int, str]
-           ) -> Iterator[tuple]:
+           ) -> Cursor:
     return db.execute(
         f'''SELECT {",".join(select_fields)} FROM {table} WHERE {key} = ?''',
         (key_value,)
     )
 
 
-def select_all(db: Connection, table: str, select_fields: List[str]) -> Iterator[tuple]:
+def select_all(db: Connection, table: str, select_fields: List[str]) -> Cursor:
     return db.execute(f'''SELECT {",".join(select_fields)} FROM {table}''')
 
 
@@ -114,14 +125,18 @@ def vacuum(db: Connection):
 
 
 def check_errors(db: Connection, table: str) -> List[tuple]:
-    if (table := table.upper()) == "SUBMISSIONS":
-        id_errors: List[tuple] = select(db, table, ["*"], "ID", 0)
-        author_errors: List[tuple] = select(db, table, ["*"], "AUTHOR", "")
-        title_errors: List[tuple] = select(db, table, ["*"], "TITLE", "")
-        date_errors: List[tuple] = select(db, table, ["*"], "UDATE", "")
-        file_url_errors: List[tuple] = select(db, table, ["*"], "FILELINK", "")
+    if (table := table.upper()) in ("SUBMISSIONS", "JOURNALS"):
+        errors: List[tuple] = []
+        errors.extend(select(db, table, ["*"], "ID", 0).fetchall())
+        errors.extend(select(db, table, ["*"], "AUTHOR", "").fetchall())
+        errors.extend(select(db, table, ["*"], "TITLE", "").fetchall())
+        errors.extend(select(db, table, ["*"], "UDATE", "").fetchall())
 
-        return list(set(id_errors + author_errors + title_errors + date_errors + file_url_errors))
+        if table == "SUBMISSIONS":
+            errors.extend(select(db, table, ["*"], "FILELINK", "").fetchall())
+            errors.extend(select(db, table, ["*"], "FILESAVED", "").fetchall())
+
+        return sorted(set(errors), key=lambda s: s[0])
     else:
         return []
 
@@ -146,6 +161,17 @@ def make_database(db: Connection):
         PRIMARY KEY (ID ASC));"""
     )
 
+    # Create journals table
+    db.execute(
+        """CREATE TABLE IF NOT EXISTS JOURNALS
+        (ID INT UNIQUE NOT NULL,
+        AUTHOR TEXT NOT NULL,
+        TITLE TEXT,
+        UDATE DATE NOT NULL,
+        CONTENT TEXT,
+        PRIMARY KEY (ID ASC));"""
+    )
+
     # Create users table
     db.execute(
         """CREATE TABLE IF NOT EXISTS USERS
@@ -155,6 +181,7 @@ def make_database(db: Connection):
         SCRAPS TEXT,
         FAVORITES TEXT,
         MENTIONS TEXT,
+        JOURNALS TEXT,
         PRIMARY KEY (USERNAME ASC));"""
     )
 
@@ -171,6 +198,7 @@ def make_database(db: Connection):
     # Add settings
     insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["USRN", "0"], False)
     insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["SUBN", "0"], False)
+    insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["JRNN", "0"], False)
     insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["LASTUPDATE", "0"], False)
     insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["LASTSTART", "0"], False)
     insert(db, "SETTINGS", ["SETTING", "SVALUE"], ["COOKIES", "{}"], False)
