@@ -1,3 +1,4 @@
+from datetime import datetime
 from math import log10
 from os import get_terminal_size
 from os import makedirs
@@ -11,18 +12,22 @@ from typing import Tuple
 from typing import Union
 
 from faapi import FAAPI
+from faapi import Journal
 from faapi import Sub
 from faapi import SubPartial
 from filetype import guess_extension
 
 from .database import Connection
 from .database import insert
+from .database import keys_journals
 from .database import keys_submissions
 from .database import keys_users
 from .database import select
+from .database import select_all
 from .database import tiered_path
 from .database import update
 from .settings import setting_read
+from .settings import setting_write
 
 
 class UnknownFolder(Exception):
@@ -40,18 +45,18 @@ def user_clean_name(user: str) -> str:
     return str(re_sub(r"[^a-zA-Z0-9\-.~,]", "", user.lower().strip()))
 
 
-def submission_clean_title(title: str) -> str:
+def clean_title(title: str) -> str:
     return str(re_sub(r"[^\x20-\x7E]", "", title.strip()))
 
 
 def user_check(db: Connection, user: str, field: str, value: str) -> bool:
-    field_value: List[str] = next(select(db, "USERS", [field], "USERNAME", user))[0].split(",")
+    field_value: List[str] = select(db, "USERS", [field], "USERNAME", user).fetchone()[0].split(",")
 
     return value in filter(len, field_value)
 
 
 def user_add(db: Connection, user: str, field: str, add_value: str):
-    field_old: List[str] = next(select(db, "USERS", [field], "USERNAME", user))[0].split(",")
+    field_old: List[str] = select(db, "USERS", [field], "USERNAME", user).fetchone()[0].split(",")
     field_old = list(filter(len, field_old))
 
     if add_value in field_old:
@@ -62,6 +67,11 @@ def user_add(db: Connection, user: str, field: str, add_value: str):
     db.commit()
 
 
+def user_edit(db: Connection, user: str, fields: List[str], new_values: List[str]):
+    update(db, "USERS", fields, new_values, "USERNAME", user)
+    db.commit()
+
+
 def user_new(db: Connection, user: str):
     insert(db, "USERS", keys_users, [user] + [""] * (len(keys_users) - 1), replace=False)
     db.commit()
@@ -160,25 +170,82 @@ def submission_download(api: FAAPI, db: Connection, sub_id: int) -> bool:
     return True
 
 
+def journal_check(db: Connection, journal_id: int) -> bool:
+    return bool(select(db, "JOURNALS", ["ID"], "ID", journal_id).fetchone())
+
+
+def journal_save(db: Connection, journal: Journal):
+    insert(db, "JOURNALS", keys_journals,
+           [journal.id, journal.author, journal.title, journal.date, journal.content])
+
+
+def users_download(api: FAAPI, db: Connection, users: List[str], folders: List[str]):
+    for user, folder in ((u, f) for u in users for f in folders):
+        print(f"Downloading: {user}/{folder}")
+        tot, fail = user_download(api, db, user, folder)
+        print("Items downloaded:", tot)
+        print("Items failed:", fail) if fail else None
+
+
+def users_update(api: FAAPI, db: Connection, users: List[str] = None, folders: List[str] = None, stop: int = 1):
+    tot: int = 0
+    fail: int = 0
+    for user, user_folders in select_all(db, "USERS", ["USERNAME", "FOLDERS"]):
+        if users and user not in users:
+            continue
+        elif not api.user_exists(user):
+            print(f"User {user} not found")
+            new_folders = list(map(lambda f: "!" + f if f in ("gallery", "scraps") else f, user_folders.split(",")))
+            user_edit(db, user, ["FOLDERS"], new_folders)
+            continue
+        for folder in user_folders.split(","):
+            if folders and folder not in folders:
+                continue
+            if folder.lower().startswith("mentions"):
+                print(f"Unsupported: {user}/{folder}")
+                continue
+            print(f"Downloading: {user}/{folder}")
+            tot_tmp, fail_tmp = user_download(api, db, user, folder, stop)
+            tot += tot_tmp
+            fail += fail_tmp
+    print("Items downloaded:", tot)
+    print("Items failed:", fail) if fail else None
+    setting_write(db, "LASTUPDATE", str(datetime.now().timestamp()))
+
+
 def user_download(api: FAAPI, db: Connection, user: str, folder: str, stop: int = 0) -> Tuple[int, int]:
-    subs_total: int = 0
-    subs_failed: int = 0
+    items_total: int = 0
+    items_failed: int = 0
+    items_type: str = "sub"
     page: Union[int, str] = 1
     page_n: int = 0
     user = user_clean_name(user)
     space_term: int = get_terminal_size()[0]
     found_subs: int = 0
 
-    downloader: Callable[[str, Union[str, int]], Tuple[List[SubPartial], Union[int, str]]] = lambda *x: ([], 0)
-    if folder == "gallery":
+    if not api.user_exists(user):
+        print(f"User {user} not found")
+        return 0, 0
+
+    downloader: Callable[
+        [str, Union[str, int]],
+        Tuple[Union[List[SubPartial], List[Journal]], Union[int, str]]
+    ]
+    if folder.endswith("!"):
+        print(f"{folder} disabled")
+        return 0, 0
+    elif folder == "gallery":
         downloader = api.gallery
     elif folder == "scraps":
         downloader = api.scraps
     elif folder == "favorites":
         page = "next"
         downloader = api.favorites
+    elif folder == "journals":
+        items_type = "journal"
+        downloader = api.journals
     else:
-        UnknownFolder(folder)
+        raise UnknownFolder(folder)
 
     user_new(db, user)
     user_add(db, user, "FOLDERS", folder.lower())
@@ -187,28 +254,36 @@ def user_download(api: FAAPI, db: Connection, user: str, folder: str, stop: int
         page_n += 1
         space_title: int = space_term - 29 - (int(log10(page_n)) + 1)
         print(f"{page_n}    {user[:space_title]} ...", end="", flush=True)
-        user_subs, page = downloader(user, page)
-        if not user_subs:
+        items, page = downloader(user, page)
+        if not items:
             print("\r" + (" " * 31), end="\r", flush=True)
-        for i, sub in enumerate(user_subs, 1):
+        for i, item in enumerate(items, 1):
             print(
-                f"\r{page_n}/{i:02d} {sub.id:010d} " +
-                f"{submission_clean_title(sub.title)[:space_title]:<{space_title}} ",
+                f"\r{page_n}/{i:02d} {item.id:010d} " +
+                f"{clean_title(item.title)[:space_title]:<{space_title}} ",
                 end="",
                 flush=True
             )
-            if not sub.id:
-                subs_failed += 1
+            if not item.id:
+                items_failed += 1
                 print(f"[{'ID ERROR':^10}]")
-            elif user_check(db, user, folder.upper(), str(sub.id).zfill(10)):
+            elif user_check(db, user, folder.upper(), str(item.id).zfill(10)):
                 print(f"[{'FOUND':^10}]")
                 if stop and (found_subs := found_subs + 1) >= stop:
-                    return subs_total, subs_failed
-            elif submission_check(db, sub.id):
+                    return items_total, items_failed
+            elif items_type == "sub" and submission_check(db, item.id):
                 print(f"[{'FOUND':^10}]")
-                user_add(db, user, folder.upper(), str(sub.id).zfill(10))
-            elif submission_download(api, db, sub.id):
-                user_add(db, user, folder.upper(), str(sub.id).zfill(10))
-                subs_total += 1
+                user_add(db, user, folder.upper(), str(item.id).zfill(10))
+            elif items_type == "sub" and submission_download(api, db, item.id):
+                user_add(db, user, folder.upper(), str(item.id).zfill(10))
+                items_total += 1
+            elif items_type == "journal" and journal_check(db, item.id):
+                print(f"[{'FOUND':^10}]")
+                user_add(db, user, folder.upper(), str(item.id).zfill(10))
+            elif items_type == "journal":
+                print(f"[{'#' * 10}]")
+                journal_save(db, item)
+                user_add(db, user, folder.upper(), str(item.id).zfill(10))
+                items_total += 1
 
-    return subs_total, subs_failed
+    return items_total, items_failed
